<?php

namespace Emulator\Messages\Incoming;

use Emulator\Encoding\ByteEncoding;

class ClientMessage {
    private readonly array $packet;
    private readonly int $header;
    private int $readerIndex;

    public function __construct(array $packet) {
        $this->packet = $packet;
        $this->readerIndex = 0;
        $this->header = $this->readInt16();
    }

    private function readBytes(int $amount): array {
        $bytes = [];
        for ($i = 0; $i < $amount && $this->readerIndex < count($this->packet); $i++) {
            $bytes[$i] = $this->packet[$this->readerIndex++];
        }
        return $bytes;
    }

    public function readInt16(): int {
        return ByteEncoding::getInt16($this->readBytes(2));
    }

    public function readInt32(): int {
        return ByteEncoding::getInt32($this->readBytes(4));
    }

    public function readBoolean(): bool {
        return $this->packet[$this->readerIndex++] === 1;
    }

    public function readString(): string {
        return call_user_func_array('pack', array_merge(['C*'], $this->readBytes($this->readInt16())));
    }

    public function getHeader(): int{
        return $this->header;
    }
}