<?php

namespace Emulator\Network\Game\Sessions;

use Emulator\Encoding\ByteEncoding;
use Emulator\Messages\Outgoing\IMessageComposer;
use Emulator\Messages\Outgoing\ServerMessage;
use OpenSwoole\Http\Request;
use OpenSwoole\WebSocket\Server;

readonly class Session {
    private Server $server;
    private Request $request;

    public function __construct(Server $server, Request $request) {
        $this->server = $server;
        $this->request = $request;
    }

    public function sendComposer(IMessageComposer $composer): void {
        $message = new ServerMessage($composer->getHeader()->value);
        $composer->compose($message);
        $data = array_merge(ByteEncoding::int32ToArray(count($message->getPacket())), $message->getPacket());

        $this->server->push($this->request->fd, pack('C*', ...$data), Server::WEBSOCKET_OPCODE_BINARY);
    }
}