<?php

namespace Emulator\Utilities;

use InvalidArgumentException;

class Dictionary {
    private array $arr;
    public function __construct(private readonly string $valueType) {
        $this->arr = [];
    }

    public function add(mixed $key, mixed $value): void {
        if (!is_int($key) && !is_string($key) && !TypeChecker::isEnum($key)) {
            throw new InvalidArgumentException('Key must be an integer, string or an enum');
        }

        if (!$value instanceof $this->valueType) {
            throw new InvalidArgumentException("Value must be an instance of {$this->valueType}");
        }

        if (TypeChecker::isEnum($key)) {
            $key = $key->value;
        }

        if (array_key_exists($key, $this->arr)) {
            throw new InvalidArgumentException("Key $key already exists");
        }

        $this->arr[$key] = $value;
    }

    public function get($key): mixed {
        if (TypeChecker::isEnum($key)) {
            $key = $key->value;
        }

        return $this->arr[$key] ?? null;
    }

    public function containsKey($key): bool {
        if (TypeChecker::isEnum($key)) {
            $key = $key->value;
        }

        return array_key_exists($key, $this->arr);
    }

    public function count(): int {
        return count($this->arr);
    }

    public function remove($key): void {
        if (TypeChecker::isEnum($key)) {
            $key = $key->value;
        }

        unset($this->arr[$key]);
    }
}